% Create an Optickle model of Advanced LIGO
%
% opt = optLIGO2
 
% lPRC = lPR + (lIX + lIY) / 2;
% lSRC = lSR + (lIX + lIY) / 2;
% lasy = lIX - lIY;

% dTune - the SRCL detuning distance
%
% The detuning phase used in the Bench is the round trip phase:
%   ifo.Optics.SRM.Tunephase = 2 * k * dTune
% where k = 2 * pi / lambda
%
% Buonanno & Chen gr-qc/0102012 use the one-way phase, given by:
%   Chen.Tunephase = pi / 2 - k * dTune
%  which is similar to the total offset on the SR set with:
%   opt = setPosOffset(opt, nSR, opt.lambda / 4 + dTune);
%  since
%   Chen.Tunephase = k * (lambda / 4 + dTune)
%
%  The lambda / 4 is there to get close to the resonance, and 
%  dTune to set the detuning frequency.  For the LIGO2 detector,
%  the detuning frequency is roughly given by:
%    detuneFreq =?= tan(k * (lambda / 4 + dTune)) * (-13 Hz)
%
%  (lPR - lSR) is set to resonate the upper 180 MHz SB.  Using
%  -dTune instead resonates the lower 180 MHz SB, which destroys
%  the detectors RSE behaviour.

function opt = optLCGT

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Detector Parameters

  % detector geometry (distances in meters)
  Ri = 2076;		% radius of curvature of input mirrors (IX and IY)
  Re = 2076;		% radius of curvature of end mirrors (EX and EY)
  Rr = 1442;		% radius of curvature of recycling mirrors (PR and SR)

  lPR = 20.01797835;     % optimized +/-f2, distance from PR to BS
  lSR = 19.97941315;     % optimized +/-f1, distance from SR to BS
  %  dTune = 1e-8;		% SR micro detuning (see note)
  dTune = 7.3889e-9*0;		% SR micro detuning (see note)

  %%% optical parameters
  %
  
  % Typical loss at test mass
  Loss = 10e-6;
  ARLoss = 1000e-6;
  
  % Arm cavity Finesse and imbalance
  Titm = 0.00400; %input lossless value
  dTitm = Titm/100*0;

  % the loss on EX and EY is used to set the contrast defect
  dLoss = 0;
  
  % having 1% reflection at REFL
  dTprm = 0; 

  
  lIX = 28.716274425;   % distance from BS to IX
  lIY = 21.221462974;         % distance from BS to IY
  
  lEX = 3006.69;		% length of the X arm
  lEY = 3006.69;        % length of the Y arm
  
  % mechanical parameters
  wI = 2 * pi * 0.8;		% resonance frequency of input mirror (rad/s)
  mI = 30;			% mass of input mirror (kg)
  wE = 2 * pi * 0.8;		% resonance frequency of end mirror (rad/s)
  mE = 30;			% mass of end mirror (kg)

  % input beam parameters
  %Pin = 75.3761*1.228092608130379e+01;			% input power (W)
  %Pin = 75.3761;			% input power (W)
  %f1 = 10e6;			% first modulation frequency
  %f2 = f1 * 6;			% second modulation frequency
  %g1 = 0.1;			% first modulation depth
  %g2 = 0;			% second modulation depth
  %Nmod1 = 1;			% first modulation order
  %Nmod2 = 1;			% second modulation order

  % construct modulation vectors
  %n1 = (-Nmod1:Nmod1)';
  %n2 = (-Nmod2:Nmod2)';
  %N1 = length(n1);
  %N2 = length(n2);
  %vMod = zeros(N1 * N2, 2);
  %for n = 1:N1
  %  nn = N1 * (n - 1) + (1:N2);
  %  ff = n1(n) * f1 + n2 * f2;
  %  aa = sqrt(Pin) * bessel(n1(n), g1) * bessel(n2, g2);
  %  vMod(nn, :) = [ff, aa];
  %end
  %vMod = sortrows(vMod, 1);
  %vMod.^2
  
  % create an empty model
  %opt = Optickle;
  
  
  % RF component vector
  Pin = 75;
  vMod = (-1:1)';
  f1 = 10e6;			% first modulation frequency
  %f2 = f1 * 6;			% second modulation frequency
  %fMod = 10e6;
  vFrf = f1 * vMod;
  
  % create model
  opt = Optickle(vFrf);
  
  % add a source
  opt = addSource(opt, 'Laser', sqrt(Pin) * (vMod == 0));
  
  
  % add an AM modulator (for intensity control, and intensity noise)
  %   opt = addModulator(opt, name, cMod)
  opt = addModulator(opt, 'AM', 1);
  opt = addLink(opt, 'Laser', 'out', 'AM', 'in', 0);

  % add an PM modulator (for frequency control and noise)
  opt = addModulator(opt, 'PM', i);
  opt = addLink(opt, 'AM', 'out', 'PM', 'in', 0);

  % add an RF modulator
  %   opt = addRFmodulator(opt, name, fMod, aMod)
  gamma = 0.00001;
  opt = addRFmodulator(opt, 'Mod1', f1, i * gamma);
  opt = addLink(opt, 'PM', 'out', 'Mod1', 'in', 1);
  
  
  if 0
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add a Field Source

  % make the modulation vectors and add a source
  %[opt, nSrc] = addSource(opt, 'Source', vMod);

  % add an AM modulator
  [opt, nAM] = addModulator(opt, 'AM', 1);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Mirrors
  % The argument list for addMirror is:
  % [opt, sn] = addMirror(opt, name, aio, Chr, Thr, Lhr, Rar)
  % type "help addMirror" for more information
  
  end
  
  
  % add recycling mirrors
  [opt, nPR] = addMirror(opt, 'PR', 0, 1 / Rr, 0.2 - Loss + dTprm, Loss, ARLoss, 0);
  [opt, nSR] = addMirror(opt, 'SR', 0, 1 / Rr, 0.23 - Loss, Loss, ARLoss*0, 0);

  % add beam-splitter
  [opt, nBS] = addBeamSplitter(opt, 'BS', 45, 0, 0.5 - 10*Loss/2, 10*Loss, ARLoss*2*0, 0);

  % add arm mirrors (a.k.a. test-masses)
  [opt, nBSPX] = addBeamSplitter(opt, 'BSPX', 45, 0, 0.999, Loss, ARLoss*3 + 750e-6);
  [opt, nIX] = addMirror(opt, 'IX', 0, 1 / Ri, Titm + dTitm - Loss, Loss, ARLoss, 20e-6*0.15*0);
  [opt, nBSPY] = addBeamSplitter(opt, 'BSPY', 45, 0, 0.999, Loss, ARLoss*3 + 750e-6);
  [opt, nIY] = addMirror(opt, 'IY', 0, 1 / Ri, Titm - dTitm - Loss, Loss, ARLoss, 20e-6*0.15*0);
  [opt, nEX] = addMirror(opt, 'EX', 0, 1 / Re, 50e-6 -Loss, Loss + dLoss, ARLoss, 20e-6*0.15*0);
  [opt, nEY] = addMirror(opt, 'EY', 0, 1 / Re, 50e-6 -Loss, Loss - dLoss, ARLoss, 20e-6*0.15*0);

  % OMC mirrors
  [opt, nOMCa] = addMirror(opt, 'OMCa', 0, 1, pi/500*0,0,0);
  [opt, nOMCb] = addMirror(opt, 'OMCb', 0, 1, pi/500*0,0,0);
  
  % set mechanical transfer-functions for the test-masses (m/N)
  dampedRes = [-0.2 + 1i, -0.2 - 1i];

  %opt = setMechTF(opt, nIX, zpk([], wI * dampedRes, 1 / mI));
  %opt = setMechTF(opt, nEX, zpk([], wE * dampedRes, 1 / mE));
  %opt = setMechTF(opt, nIY, zpk([], wI * dampedRes, 1 / mI));
  %opt = setMechTF(opt, nEY, zpk([], wE * dampedRes, 1 / mE));

  opt = setMechTF(opt, nIX, zpk([], wI * dampedRes, 2 / mI));
  opt = setMechTF(opt, nEX, zpk([], wE * dampedRes, 2 / mE));
  opt = setMechTF(opt, nIY, zpk([], wI * dampedRes, 2 / mI));
  opt = setMechTF(opt, nEY, zpk([], wE * dampedRes, 2 / mE));
  
  % set position offsets for SR
  opt = setPosOffset(opt, nSR, opt.lambda / 4 + dTune);

  % set position offsets for DARM for DCreadout
%  opt = setPosOffset(opt, nEX, 6.5e-12); %%%
%  opt = setPosOffset(opt, nEY, -6.5e-12); %%%
%  opt = setPosOffset(opt, nEX, -5.5556e-6*1.064e-6); %%%
%  opt = setPosOffset(opt, nEY, 5.5556e-6*1.064e-6); %%%
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Field Sinks (needed by the probes)

  [opt, nREFL] = addSink(opt, 'REFL');
  [opt, nAS] = addSink(opt, 'AS');
  [opt, nPOX] = addSink(opt, 'POX');
  [opt, nPOY] = addSink(opt, 'POY');
  [opt, nTRX] = addSink(opt, 'TRX');
  [opt, nTRY] = addSink(opt, 'TRY');
  [opt, nOMC] = addSink(opt, 'OMC');
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Links

  % link input to PR
%  opt = addLink(opt, nSrc, 'out', nAM, 'in', 1);
%  opt = addLink(opt, nAM, 'out', nPR, 'bk', 10);
%  opt = addLink(opt, nSrc, 'out', nPR, 'bk', 10);
  opt = addLink(opt, 'Mod1', 'out', 'PR', 'bk', 2);

  % link BS A-side inputs to PR and SR front outputs
  opt = addLink(opt, nPR, 'fr', nBS, 'frA', lPR);
  opt = addLink(opt, nSR, 'fr', nBS, 'bkA', lSR);

  % link BS A-side outputs to and IX and IY back inputs
  opt = addLink(opt, nBS, 'frA', nBSPY, 'frA', lIY); % for vacuum injection on POY
  opt = addLink(opt, nBS, 'bkA', nBSPX, 'frA', lIX); % for vacuum injection on POX
  opt = addLink(opt, nBSPY, 'bkA', nIY, 'bk', 0);
  opt = addLink(opt, nBSPX, 'bkA', nIX, 'bk', 0);

  % link BS B-side inputs to and IX and IY back outputs
  opt = addLink(opt, nIY, 'bk', nBSPY, 'bkB', 0);
  opt = addLink(opt, nIX, 'bk', nBSPX, 'bkB', 0);
  opt = addLink(opt, nBSPY, 'frB', nBS, 'frB', lIY);
  opt = addLink(opt, nBSPX, 'frB', nBS, 'bkB', lIX);  % link BS B-side outputs to PR and SR front inputs
  opt = addLink(opt, nBS, 'frB', nPR, 'fr', lPR);
  opt = addLink(opt, nBS, 'bkB', nSR, 'fr', lSR);

  % link the arms
  opt = addLink(opt, nIX, 'fr', nEX, 'fr', lEX);
  opt = addLink(opt, nEX, 'fr', nIX, 'fr', lEX);

  opt = addLink(opt, nIY, 'fr', nEY, 'fr', lEY);
  opt = addLink(opt, nEY, 'fr', nIY, 'fr', lEY);

  % link the OMC
  %opt = addLink(opt, nSR, 'bk', nOMCa, 'bk', 5);
  %opt = addLink(opt, nOMCa, 'fr', nOMCb, 'fr', 0.22);
  %opt = addLink(opt, nOMCb, 'fr', nOMCa, 'fr', 0.22);
  
  % link the detectors
  opt = addLink(opt, nPR, 'bk', nREFL, 'in', 5);
  opt = addLink(opt, nSR, 'bk', nAS, 'in', 5);
  opt = addLink(opt, nIX, 'po', nPOX, 'in', 5);
  opt = addLink(opt, nIY, 'po', nPOY, 'in', 5);
  opt = addLink(opt, nEX, 'bk', nTRX, 'in', 5);
  opt = addLink(opt, nEY, 'bk', nTRY, 'in', 5);
%  opt = addLink(opt, nOMCb, 'bk', nAS, 'in', 5);
  opt = addLink(opt, nOMCa, 'bk', nOMC, 'in', 5);

  % tell Optickle to use this cavity basis
  %opt = setCavityBasis(opt, 'IX', 'EX');
  %opt = setCavityBasis(opt, 'IY', 'EY');
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Probes

  % AS signals (anti-symmetric port)
  %opt = addProbeAt(opt, 'AS DC', nAS, 'in',  0,   0);		% DC
  %opt = addProbeAt(opt, 'AS I1', nAS, 'in', f1,   0);		% f1 demod I
  %opt = addProbeAt(opt, 'AS Q1', nAS, 'in', f1,  90);		% f1 demod Q

 %phi = 144;
  phi = 85;
  opt = addReadout(opt, 'AS', [f1, phi]);
  
  % add unphysical intra-cavity probes
  opt = addProbeIn(opt, 'PR_DC', 'PR', 'fr', 0, 0);
  opt = addProbeIn(opt, 'IX_DC', 'IX', 'fr', 0, 0);
  opt = addProbeIn(opt, 'EX_DC', 'EX', 'fr', 0, 0);

  
  